"""
Proccess Meraki Data Model to find all devices with in an org and save informato to model
"""
import asyncio
import logging
import threading
import time
from datetime import datetime
from merakitools import const, lib, model
from .processor import proccess_network


class Org_device_proccesssor(threading.Thread):
	"""
    Setup meraki devices in data model in memory
    """
	
	def __init__(self, org_id):
		"""
        Init of Device Sync Sync Pocessor
        Args:
            org_id:ID or Org to be proccessed
        """
		threading.Thread.__init__(self)
		self.org_id = org_id
		self.networks = {}
	
	def run(self):
		start_time = time.perf_counter()
		threading.currentThread().setName(
			model.meraki_nets[self.org_id].name)
		asyncio.run(self._get_org_networks())
		if const.appcfg.debug:
			print(f'\tOrgName:{model.meraki_nets[self.org_id].name} sync '
			      f'Started at: {start_time} '
			      f'Thread PID:{threading.currentThread().native_id}')
		
		devices = asyncio.run(self._get_org_devices())
		for device in devices:
			model.meraki_nets[self.org_id].devices.update(
					{device['serial']: model.Device(device, self.org_id)})
		
		asyncio.run(self._switch_procssor())
		
		model.meraki_nets[self.org_id].syncruntime = \
			time.perf_counter() - start_time
		model.meraki_nets[self.org_id].lastsync = datetime.utcnow()
		lib.store_cache(self.org_id,model.meraki_nets[self.org_id],'device_config')
		print(
			f'{lib.bc.OKBLUE}OrgName:{model.meraki_nets[self.org_id].name} finished device sync{lib.bc.ENDC}')
	
	async def _get_org_networks(self):
		with lib.MerakiAsyncApi() as sdk:
			networks = await sdk.organizations.getOrganizationNetworks(
				self.org_id)
		for network in networks:
			self.networks.update({network['id']: network['tags']})
	
	async def _get_org_devices(self):
		with lib.MerakiAsyncApi() as sdk:
			return await sdk.organizations.getOrganizationDevices(
				self.org_id)
	
	def _setup_config_model(self, serial):
		device = model.meraki_nets[self.org_id].devices[serial]
		is_golden = False
		port_ids = {}
		if const.appcfg.tag_golden in self.networks[device.net_id]:
			is_golden = True
		if const.appcfg.tag_target in self.networks[device.net_id]:
			for port in device.config:
				for port_tags in port['tags']:
					if port_tags in const.appcfg.switch_port_tags:
						if is_golden:
							port.pop('portId')
							model.config_manager[
								const.appcfg.tag_golden].golden_port_config.update(
									{port_tags: port})
						else:
							if serial in model.config_manager[
									const.appcfg.tag_golden].target_switches[
									port_tags].keys():
								model.config_manager[
									const.appcfg.tag_golden].target_switches[
									port_tags][serial].append(
										port['portId'])
							else:
								model.config_manager[
									const.appcfg.tag_golden].target_switches[
									port_tags].update({serial: [port['portId']]})
								
								
	
	async def _switch_procssor(self):
		"""
        Function proccess Each swithin in an org and pulls configuation infroamtion down for each port on the switch
        The function will also place the port the data_model that is orginizaed by network tag, then poprt configuraiton tage
        Returns:

        """
		with lib.MerakiAsyncApi() as sdk:
			for device in model.meraki_nets[self.org_id].devices:
				serial = model.meraki_nets[self.org_id].devices[
					device].serial
				if model.meraki_nets[self.org_id].devices[
					device].product == 'switch':
					print(f'{lib.bc.OKBLUE} Orginization: {model.meraki_nets[self.org_id].name}-{lib.bc.WARNING}starting switch {serial} config sync{lib.bc.ENDC}')
					model.meraki_nets[self.org_id].devices[
						serial].config = await sdk.switch.getDeviceSwitchPorts(
							serial)
					self._setup_config_model(serial)
					print(f'{lib.bc.OKBLUE} Orginization: {model.meraki_nets[self.org_id].name}-{lib.bc.WARNING}finished switch {serial} config sync {lib.bc.ENDC}')



"""
Configuration Tag Manager data model
"""


class Tag_Manager:
	"""
		manages tag combinations for switchport configuration
	"""
	
	def __init__(self, golden_tag, network_target_tag, port_tags):
		self.network_golden_tag = golden_tag
		self.network_target_tag = network_target_tag
		self.port_tag = port_tags
		self.golden_port_config = {}
		self.approve_golden_config = {}
		self.target_switches = {}
		self.org_id = golden_tag
		self.name = "PortConfigurations"



from ._getorginizations import getOrginizationsAll, \
	getOrginization,set_orginization_by_id,set_orginizations_by_name
from ._orgsyncprocessor import Orgsyncprocessor
from ._validateorginization import Validateorginization
from ._orgdeviceprocessor import Org_device_proccesssor
from ._validatedeviceconfig import Validatedevice


"""
Meraki Network Object
"""
from merakitools import products

class MNET:
    """
    Network Object for each Meraki dashbaord network
    """
    #Network Object for each dashboard network
    def __init__(self,net):
        self.net_id = net['id']
        self.org_id = net['organizationId']
        self.name = net['name']
        self.tags = net['tags']
        self.products = net['productTypes']
        self.syncruntime = None
        self.lastsync = None
        self.cached = False
        self.change_log_ts = None
        self.supported = products.supported
        self.dashboard = self._get_config()
        self.functions = self._product_functions()
    def _get_config(self):
        temp_list = {}
        for product in self.supported:
            tmethod = getattr(products, product)
            method = getattr(tmethod,product)
            temp = method()
            t_return = {product: temp}
            temp_list.update(t_return)
        return temp_list
    def _product_functions(self):
        temp_list = {}
        for product in self.supported:
            #method = getattr(self,dashboard[product])
            _module = dir(self.dashboard[product])
            update = [_funct.split('_')[1] for _funct in _module if
                                  _funct.startswith('Update_')]
            get = [_funct.split('_')[1] for _funct in _module if
                               _funct.startswith('Get_')]
            t_return = {product:{'update': update, 'get': get}}
            temp_list.update(t_return)
        return temp_list

    def __dir__(self):
        return self.__dict__.keys()




import asyncio
import click
from merakitools.main import run
from merakitools import const

@click.group()
def cli():
    """ Tasks to start configuration of Meraki Switches by tag based on golden configurations """
    pass


@click.command(name="port-config",help='Meraki Switch port configuration based on tag')
@click.option('-f','--configfile', help='Config File')
def port_config(configfile=None,):
    if configfile is not None:
        run(configfile, 'device_config')

cli.add_command(port_config)
